#!/bin/bash

# -----------------------------------------------------------------
# variables
# -----------------------------------------------------------------

# sourcing /etc/default/epages6
. /etc/default/epages6
# provides following variables/functions:
export MONGOD_CONF EPAGESJ EPAGESJ_CONFIG
export mongod_name test_version

BN=${0##*/}
DN=$(cd $(dirname $0) && pwd)
OPENSSL_CMD=openssl
MONGO_CMD=mongo
INI_CONFIG=/var/epages/ini-config.sh

# used scripts
CREATE_USER_SH="$DN/mongo_create_user.sh"
KEYFILE_SH="$DN/mongo_keyfile.sh"

# mongo daemon name (used for stop/start)
MONGOD=$(mongod_name)

# argument defaults
MONGO_CONF_FILE=${MONGOD_CONF:-"/etc/$MONGOD.conf"}
EPAGESJ=${EPAGESJ:-$EPAGES/j}
EPAGESJ_CONF=${EPAGESJ_CONFIG:-$EPAGESJ/conf}/epagesj.conf

# -----------------------------------------------------------------
# arguments
# -----------------------------------------------------------------

usage() {
  [[ $# -gt 0 ]] && echo -e "Error:\n  $@"
  echo "Usage:"
  echo "  $0 [-f keyfile] [-c mongodb.conf] [-j epagesj.conf] [-h]"
  echo "Description:"
  echo "  enable mongodb for authorization by creating db users"
  echo "  and configure them in a keyfile to use"
  echo "Options:"
  echo "  -f: auth keyfile to create and use (default: <dbpath>/mongodb-keyfile)"
  echo "  -c: mongodb config file (default: $MONGO_CONF_FILE)"
  echo "  -j: epagesj config file (default: $EPAGESJ_CONF)"
  echo "  -h: show usage"
  echo "Example:"
  echo "  $0 -f /usr/local/etc/mongo_secret -c /etc/mongo_xyz.conf"
  exit 1
}

#get options from command line
unset KEYFILE
while getopts f:c:j: char; do
case $char in
    f) KEYFILE=$OPTARG ;;
    c) MONGO_CONF_FILE=$OPTARG ;;
    j) EPAGESJ_CONF=$OPTARG ;;
    *) usage ;;
  esac
done
shift `expr $OPTIND - 1`

[[ $# -ne 0 ]] && usage "$BN: No extra arguments allowed: $@"
[[ -f "$MONGO_CONF_FILE" ]] || usage "$BN: No such file: $MONGO_CONF_FILE"
[[ -f "$EPAGESJ_CONF" ]] || usage "$BN: No such file: $EPAGESJ_CONF"

#check if openssl is installed
type $OPENSSL_CMD || exit 1

#get mongo version or die
MONGO_VERSION=$($MONGO_CMD --version) || exit 1
MONGO_VERSION=${MONGO_VERSION##*:}

#how to enable auth:
#http://docs.mongodb.org/manual/tutorial/enable-authentication-without-bypass/

# -----------------------------------------------------------------
# get/update mongodb information on epagesj.conf
# -----------------------------------------------------------------

password=$($INI_CONFIG -f "$EPAGESJ_CONF" -s Mongo -k password -G)
if [[ -z $password ]] ; then
  # epagesj does not like [+/] in mongo passwords
  password=$($OPENSSL_CMD rand 6 -base64 | tr +/ 00)
  $INI_CONFIG -f "$EPAGESJ_CONF" -s Mongo -k password -v "$password" -i || exit 1
fi

user=$($INI_CONFIG -f "$EPAGESJ_CONF" -s Mongo -k username -G)
if [[ -z $user ]] ; then
  user=epagesj
  $INI_CONFIG -f "$EPAGESJ_CONF" -s Mongo -k username -v "$user" -i || exit 1
fi

database=$($INI_CONFIG -f "$EPAGESJ_CONF" -s Mongo -k databaseName -G)
if [[ -z $database ]] ; then
  database=epagesj
  $INI_CONFIG -f "$EPAGESJ_CONF" -s Mongo -k databaseName -v "$database" -i || exit 1
fi

# -----------------------------------------------------------------
# create mongodb users
# -----------------------------------------------------------------

# mongo 2.6 introduces some abbreviations for roles:
# root is dbAdminAnyDatabase,userAdminAnyDatabase,clusterAdmin,readWriteAnyDatabase
# dbOwner is dbAdmin,userAdmin,readWrite
# see http://docs.mongodb.org/manual/reference/built-in-roles
declare -A ROLE
if test_version $MONGO_VERSION -lt 2.6 ; then
  ROLE[siteUserAdmin]=userAdminAnyDatabase
  ROLE[siteRootAdmin]=dbAdminAnyDatabase,userAdminAnyDatabase,clusterAdmin,readWriteAnyDatabase
  ROLE[dbOwner]=dbAdmin,userAdmin,readWrite
else
  ROLE[siteUserAdmin]=userAdminAnyDatabase
  ROLE[siteRootAdmin]=root
  ROLE[dbOwner]=dbOwner
fi

$CREATE_USER_SH -d admin -u siteUserAdmin -p "$password" -r ${ROLE[siteUserAdmin]} || exit 1
$CREATE_USER_SH -d admin -u siteRootAdmin -p "$password" -r ${ROLE[siteRootAdmin]} || exit 1
$CREATE_USER_SH -d "$database" -u "$user" -p "$password" -r ${ROLE[dbOwner]} || exit 1

# -----------------------------------------------------------------
# create keyfile and change mongo.conf and epagesj.conf
# -----------------------------------------------------------------

$KEYFILE_SH ${KEYFILE:+-f $KEYFILE} -c $MONGO_CONF_FILE || exit 1

# -----------------------------------------------------------------
# stop/start mongodb
# -----------------------------------------------------------------

service $MONGOD restart

exit $?
